


; ********************************************************************
;  Basis-Routinen fuer das Interface Transmission Protokoll Version 2
; ********************************************************************








; Funktion: Setzen der Index- und Adressregister auf die Tabellen nach RAM_AKT_CON  
;
; Aufruf: INT2_SET_ADR
; Uebergabeparameter: keine 
; Rueckgabeparameter: Register IX = D_ITP2_TAB + RAM_AKT_CON
;                              IY = C_ITP2_TAB + RAM_AKT_CON
;                              HL = RAM_CON_TAB + RAM_AKT_CON
INT2_SET_ADR:

		; Laden der Tabellenadressen
		LD	IX,D_ITP2_TAB
		LD	IY,C_ITP2_TAB
		LD	HL,RAM_CON_TAB
		;


INT2_SET_ADR_B:
		;
		RET




; Funktion: Initialisieren des Interface Transmission Protokoll (ITP) und Umschalten auf auf Lesen (RD)
;
; Aufruf: INT2_INIT_RD
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A=1 -> alles ok
;                     Register A=0 -> keine Reaktion der Schnittstelle auf die Umschaltung 
; 
INT2_INIT_RD:	

		CALL	INT2_SET_ADR


		; Ruecksetzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Lesebetrieb (RD) 
		LD	A,(HL)
		RES	BIT_DDWR2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;



INT2_INIT_RD_A:
		;
		LD	C,(IX)
		IN	A,(C)
		BIT	BIT_DDAC2,A
		JR	NZ,INT2_INIT_RD_A	; warten, bis der ATMEGA8 und
						; die CONSOLE auf RD umgeschaltet haben
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_LED_GE,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC2,A
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Read-Kommando ITP2
		LD	A,C_ITP2_RD
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;
		LD	A,1
		;
		RET




INT2_INIT_RD_C:
		LD	A,0
		;
		RET








; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
;
; Aufruf: INT2_MODE_RD
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT2_MODE_RD:	
		CALL	INT2_SET_ADR

		LD	A,(HL)
		BIT	BIT_DDWR2,A
		RET	Z			; Ruecksprung, wenn RD bereits
						; aktiv ist (DDWR=0) 

		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Initialwert-Kommandos ITP2
		LD	A,C_ITP2_IW
		OUT	(C),A
		;


		; Ruecksetzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Lesebetrieb (RD) 
		LD	A,(HL)
		RES	BIT_DDWR2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A


		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet
		;
		LD	C,(IX)
INT2_MODE_RD_A:	
		IN	A,(C)
		BIT	BIT_DDAC2,A
		JR	NZ,INT2_MODE_RD_A	; warten, bis der ATMEGA8 und
						; die CONSOLE auf RD umgeschaltet haben
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet

		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_LED_GE,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC2,A
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Read-Kommando ITP2
		LD	A,C_ITP2_RD
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;

		RET




; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
;
; Aufruf: INT2_MODE_WR
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT2_MODE_WR:	
		CALL	INT2_SET_ADR

		LD	A,(HL)
		BIT	BIT_DDWR2,A
		RET	NZ			; Ruecksprung, wenn WR bereits
						; aktiv ist (DDWR=1) 


		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Initialwert-Kommandos ITP2
		LD	A,C_ITP2_IW
		OUT	(C),A
		;


		; Setzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Schreibbetrieb (WR) 
		LD	A,(HL)
		SET	BIT_DDWR2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;


		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;


		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet
		;
		LD	C,(IX)
INT2_MODE_WR_A:	
		IN	A,(C)
		BIT	BIT_DDAC2,A
		JR	Z,INT2_MODE_WR_A		; warten, bis der ATMEGA8 und
						; die CONSOLE auf WR umgeschaltet haben
						; und dies durch Setzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		SET	BIT_LED_GE,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A


		; Grundeinstellung der Steuerleitung SDEN
		LD	A,(HL)
		RES	BIT_SRAC2,A
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		LD	C,(IY)
		; ITP2-Connector1 - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C),A

		; ITP2-Connector1 - Bitmaske - Write-Kommando ITP2
		LD	A,C_ITP2_WR
		OUT	(C),A


		; Grundeinstellung der Steuerleitung SDEN
		LD	A,(HL)
		LD	C,(IX)
		OUT	(C),A
		;

		RET




; Funktion: ITP - RECEIVE 01
;
; Aufruf: ITP2_RECEIVE_01
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A, das empfangene Byte
; 
ITP2_RECEIVE_01:
		CALL	INT2_SET_ADR

		CALL	ITP2_RECEIVE_01X
		PUSH	AF
		CALL	ITP2_RECEIVE_01X
		POP	BC
		;
		; Low-Teil in B, High-Teil in A
		;
		SLA	A
		SLA	A
		SLA	A
		SLA	A
		;
		ADD	A,B
		;
		RET



ITP2_RECEIVE_01X:

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;

		; pauschales Ruecksetzen von SRAC
		LD	A,(HL)
		RES	BIT_SRAC2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

ITP2_RECEIVE_01A:	
		IN	A,(C)
		BIT	BIT_SDEN2,A
		JR	Z,ITP2_RECEIVE_01A	; warten, bis der ATMEGA8 das Datenbyte bereit
						; gestellt hat und dies durch Setzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;

		; etwas warten, bis die Pegel sicher stehen
		NOP
		NOP
		NOP
		NOP

		; jetzt koennen die Daten aus PIO 1 Port A gelesen werden
		IN	A,(C)
		LD	B,A
		;
		LD	A,(HL)
		SET	BIT_SRAC2,A
		OUT	(C),A
		LD	(HL),A
		;
ITP2_RECEIVE_01B:	
		IN	A,(C)
		BIT	BIT_SDEN2,A
		JR	NZ,ITP2_RECEIVE_01B	; warten, bis der ATMEGA8 erkannt hat,
						; dass die Daten uebernommen wurden
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet


		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

		; Grundeinstellung der Steuerleitung SRAC
		LD	A,(HL)
		RES	BIT_SRAC2,A
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		; Rckgabe des Datenbytes (D0..D3) in Register A
		LD	A,B
		AND	00001111B
		;
		RET
	


; Funktion: ITP - SEND 01
;
; Aufruf: ITP2_SEND_01
; Uebergabeparameter: Register B - das zu sendende Byte
; Rueckgabeparameter: keine
; 
ITP2_SEND_01:
		PUSH	BC
		CALL	INT2_SET_ADR
		POP	BC

		PUSH	BC
		;
		LD	A,B
		AND	00001111B
		LD	B,A
		;
		CALL	ITP2_SEND_01X
		;
		POP	BC
		;
		SRL	B
		SRL	B
		SRL	B
		SRL	B
		;
		CALL	ITP2_SEND_01X
		;
		RET



ITP2_SEND_01X:

		; Zur Kontrolle wir die rote LED eingeschaltet
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;

		; pauschales Ruecksetzen von SDEN
		LD	A,(HL)
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;

		;
ITP2_SEND_01A:	IN	A,(C)
		BIT	BIT_SRAC2,A
		JR	NZ,ITP2_SEND_01A		; warten, bis der ATMEGA8 und
						; die CONSOLE den Grundzustand eingestellt
						; haben und dies durch Ruecksetzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		LD	A,(HL)
		AND	11110000B
		;
		OR	B			; zusammenschalten der Daten- und Steuerbits
						; in Register A
		OUT	(C),A			; Daten an Port senden
		;
		NOP
		NOP
		NOP
		;
		; Setzen von SDEN
		SET	BIT_SDEN2,A
		OUT	(C),A
		LD	(HL),A
		;
		NOP
		NOP
		;
ITP2_SEND_01_B:	IN	A,(C)
		BIT	BIT_SRAC2,A
		JR	Z,ITP2_SEND_01_B		; warten, bis der ATMEGA8 und
						; die CONSOLE die Daten uebernommen haben
						; und dies durch Setzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;

		; jetzt kann die rote LED wieder abgeschaltet werden
		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

		; Rcksetzen von SDEN
		LD	A,(HL)
		RES	BIT_SRAC2,A
		RES	BIT_SDEN2,A
		LD	C,(IX)
		OUT	(C),A
		LD	(HL),A
		;
		RET
	


