




; **********************************************************
;  Basis-Routinen fuer das Interface Transmission Protokoll
; **********************************************************





; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Lesen (RD)
;
; Aufruf: SER_MODE_RD
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT_MODE_RD:	LD	A,(RAM_D_SYS1)
		BIT	BIT_DDWR,A
		RET	Z			; Ruecksprung, wenn RD bereits
						; aktiv ist (DDWR=0) 

		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		; PIO 1 - Port A - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_ITPDA_PORT),A

		; Bitmaske Port A Daten - alles Eingaenge
		LD	A,BITMSKAE
		OUT	(C_ITPDA_PORT),A


		; PIO 1 - Port B - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_SYS1_PORT),A

		; Bitmaske Port B - Initialisierung
		; B0 = DDWR   = 0 - Ausgang
		; B1 = DDAC   = 1 - Eingang
		; B2 = SDEN   = 1 - Eingang
		; B3 = SRAC   = 1 - Eingang 
		; B4 = RAMSEL = 0 - Ausgang
		; B5 = LED_RT = 0 - Ausgang
		; B6 = LED_GE = 0 - Ausgang
		; B7 = LED_GN = 0 - Ausgang
		LD	A,00001110B
		OUT	(C_SYS1_PORT),A

		; Ruecksetzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Lesebetrieb (RD) 
		LD	A,(RAM_D_SYS1)
		RES	BIT_DDWR,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A

		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet

		; Zur Kontrolle wir die rote LED eingeschaltet
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

INT_MODE_RD_A:	IN	A,(D_SYS1_PORT)
		BIT	BIT_DDAC,A
		JR	NZ,INT_MODE_RD_A	; warten, bis der ATMEGA8 und
						; die CONSOLE auf RD umgeschaltet haben
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet

		; jetzt kann die rote LED wieder abgeschaltet werden
 		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_SRAC,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A

		; PIO 1 - Port B - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_SYS1_PORT),A

		; Bitmaske Port B - Initialisierung
		; B0 = DDWR   = 0 - Ausgang
		; B1 = DDAC   = 1 - Eingang
		; B2 = SDEN   = 1 - Eingang
		; B3 = SRAC   = 0 - Ausgang 
		; B4 = RAMSEL = 0 - Ausgang
		; B5 = LED_RT = 0 - Ausgang
		; B6 = LED_GE = 0 - Ausgang
		; B7 = LED_GN = 0 - Ausgang
		LD	A,00000110B
		OUT	(C_SYS1_PORT),A

 		LD	A,(RAM_D_SYS1)
		OUT	(D_SYS1_PORT),A

		RET




; Funktion: Umschalten des Interface Transmission Protokoll (ITP) auf Schreiben (WR)
;
; Aufruf: INT_MODE_WR
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
INT_MODE_WR:	LD	A,(RAM_D_SYS1)
		BIT	BIT_DDWR,A
		RET	NZ			; Ruecksprung, wenn WR bereits
						; aktiv ist (DDWR=1) 

		; Damit waehrend des Umschaltens nicht zwei Ausgaenge auf einer Leitung
		; aktiv sind, werden als Erstes alle Bidirektionalen Leitungen
		; als Eingang programmiert

		; PIO 1 - Port A - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_ITPDA_PORT),A

		; Bitmaske Port A Daten - alles Eingaenge
		LD	A,BITMSKAE
		OUT	(C_ITPDA_PORT),A


		; PIO 1 - Port B - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_SYS1_PORT),A

		; Bitmaske Port B - Initialisierung
		; B0 = DDWR   = 0 - Ausgang
		; B1 = DDAC   = 1 - Eingang
		; B2 = SDEN   = 1 - Eingang
		; B3 = SRAC   = 1 - Eingang 
		; B4 = RAMSEL = 0 - Ausgang
		; B5 = LED_RT = 0 - Ausgang
		; B6 = LED_GE = 0 - Ausgang
		; B7 = LED_GN = 0 - Ausgang
		LD	A,00001110B
		OUT	(C_SYS1_PORT),A

		; Setzen des DDWR-Bits und damit Anforderung
		; der Umschaltung auf Schreibbetrieb (WR) 
		LD	A,(RAM_D_SYS1)
		SET	BIT_DDWR,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A

		; nun wird auf die Rueckmeldung vom ATMEGA8 und von
		; der CONSOLE gewartet

		; Zur Kontrolle wir die rote LED eingeschaltet
		SET	BIT_LED_RT,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A

INT_MODE_WR_A:	IN	A,(D_SYS1_PORT)
		BIT	BIT_DDAC,A
		JR	Z,INT_MODE_WR_A		; warten, bis der ATMEGA8 und
						; die CONSOLE auf WR umgeschaltet haben
						; und dies durch Setzen der
						; Steuerleitung BIT_DDAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet

		; jetzt kann die rote LED wieder abgeschaltet werden
 		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_SDEN,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A

		; PIO 1 - Port B - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_SYS1_PORT),A

		; Bitmaske Port B - Initialisierung
		; B0 = DDWR   = 0 - Ausgang
		; B1 = DDAC   = 1 - Eingang
		; B2 = SDEN   = 0 - Ausgang
		; B3 = SRAC   = 1 - Eingang 
		; B4 = RAMSEL = 0 - Ausgang
		; B5 = LED_RT = 0 - Ausgang
		; B6 = LED_GE = 0 - Ausgang
		; B7 = LED_GN = 0 - Ausgang
		LD	A,00001010B
		OUT	(C_SYS1_PORT),A

 		LD	A,(RAM_D_SYS1)
		OUT	(D_SYS1_PORT),A

		; PIO 1 - Port A - MODE3 Bitbetrieb (Maske folgt)
		LD	A,0CFH
		OUT	(C_ITPDA_PORT),A

		; Bitmaske Port A Daten - alles Ausgaenge
		LD	A,BITMSKAA
		OUT	(C_ITPDA_PORT),A

		RET




; Funktion: ITP - RECEIVE 01
;
; Aufruf: ITP_RECEIVE_01
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A, das empfangene Byte
; 
ITP_RECEIVE_01:
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		RES	BIT_SRAC,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;
ITP_RECEIVE_01A:	
		IN	A,(D_SYS1_PORT)
		BIT	BIT_SDEN,A
		JR	Z,ITP_RECEIVE_01A	; warten, bis der ATMEGA8 das Datenbyte bereit
						; gestellt hat und dies durch Setzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		; jetzt koennen die Daten aus PIO 1 Port A gelesen werden
		IN	A,(D_ITPDA_PORT)
		LD	B,A			; Daten retten
		;
 		LD	A,(RAM_D_SYS1)
		SET	BIT_SRAC,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;
ITP_RECEIVE_01B:	
		IN	A,(D_SYS1_PORT)
		BIT	BIT_SDEN,A
		JR	NZ,ITP_RECEIVE_01B	; warten, bis der ATMEGA8 erkannt hat,
						; dass die Daten uebernommen wurden
						; und dies durch Ruecksetzen der
						; Steuerleitung BIT_SDEN bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		; jetzt kann die rote LED wieder abgeschaltet werden
		; ausserdem 
 		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_SRAC,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A
		;
		LD	A,B
		;
		RET
	


; Funktion: ITP - SEND 01
;
; Aufruf: ITP_SEND_01
; Uebergabeparameter: Register B - das zu sendende Byte
; Rueckgabeparameter: keine
; 
ITP_SEND_01:
		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		RES	BIT_SDEN,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;
ITP_SEND_01A:	IN	A,(D_SYS1_PORT)
		BIT	BIT_SRAC,A
		JR	NZ,ITP_SEND_01A		; warten, bis der ATMEGA8 und
						; die CONSOLE den Grundzustand eingestellt
						; haben und dies durch Ruecksetzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		LD	A,B
		OUT	(D_ITPDA_PORT),A		; Daten an Port A (Daten) senden

		; Zur Kontrolle wir die rote LED eingeschaltet

		LD	A,(RAM_D_SYS1)
		SET	BIT_LED_RT,A
		SET	BIT_SDEN,A
		OUT	(D_SYS1_PORT),A
		LD	(RAM_D_SYS1),A
		;
ITP_SEND_01_B:	IN	A,(D_SYS1_PORT)
		BIT	BIT_SRAC,A
		JR	Z,ITP_SEND_01_B		; warten, bis der ATMEGA8 und
						; die CONSOLE die Daten uebernommen haben
						; und dies durch Setzen der
						; Steuerleitung BIT_SRAC bestaetigen
		; geht was am ATMEGA8 oder an der CONSOLE schief (keine Bestaetigung d.h.
		; Endlosschleife), dann bleibt die rote LED eingeschaltet
		;
		; jetzt kann die rote LED wieder abgeschaltet werden
 		LD	A,(RAM_D_SYS1)
		RES	BIT_LED_RT,A
		RES	BIT_SDEN,A
		LD	(RAM_D_SYS1),A
		OUT	(D_SYS1_PORT),A
		;
		RET
	



